Type: Package
Package: nominatimlite
Title: Interface to the 'Nominatim' API
Version: 0.6.0
Authors@R: c(
    person("Diego", "Hernangómez", , "diego.hernangomezherrero@gmail.com", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0001-8457-4658")),
    person("Jindra", "Lacko", role = c("ctb", "rev"),
           comment = c(ORCID = "0000-0002-0375-5156")),
    person("Alex", "White", role = "ctb"),
    person("OpenStreetMap", role = "cph",
           comment = "For the data")
  )
Description: Lightweight interface to the 'OpenStreetMap' 'Nominatim' API
    <https://nominatim.org/release-docs/latest/>. Extract coordinates from
    addresses, retrieve addresses from coordinates, look up amenities and
    addresses, and return results as 'tibble' or 'sf' objects.
License: MIT + file LICENSE
URL: https://dieghernan.github.io/nominatimlite/,
        https://github.com/dieghernan/nominatimlite
BugReports: https://github.com/dieghernan/nominatimlite/issues
Depends: R (>= 4.1.0)
Imports: dplyr (>= 1.0.0), jsonlite (>= 1.7.0), sf (>= 0.9.0), tools,
        utils
Suggests: arcgeocoder, ggplot2 (>= 3.0.0), knitr, quarto, testthat (>=
        3.0.0), tibble (>= 3.0.0), tidygeocoder
VignetteBuilder: quarto
Config/Needs/website: dieghernan/gitdevr, pak, tidyverse, leaflet,
        reactable, crosstalk, tidyr, htmltools
Config/roxygen2/markdown: TRUE
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
Config/testthat/parallel: false
Copyright: Data © OpenStreetMap contributors, ODbL 1.0.
        <https://www.openstreetmap.org/copyright>
Encoding: UTF-8
LazyData: true
X-schema.org-applicationCategory: cartography
X-schema.org-keywords: r, geocoding, openstreetmap, address, nominatim,
        reverse-geocoding, rstats, shapefile, r-package, spatial, cran,
        api-wrapper, api, gis, geocoder, reverse-geocoder
NeedsCompilation: no
Packaged: 2026-06-02 15:38:13 UTC; diego
Author: Diego Hernangómez [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-8457-4658>),
  Jindra Lacko [ctb, rev] (ORCID:
    <https://orcid.org/0000-0002-0375-5156>),
  Alex White [ctb],
  OpenStreetMap [cph] (For the data)
Maintainer: Diego Hernangómez <diego.hernangomezherrero@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-03 05:10:09 UTC
Built: R 4.6.0; ; 2026-06-03 08:44:56 UTC; unix
