# Generated by roxygen2: do not edit by hand

export(bbox_to_poly)
export(geo_address_lookup)
export(geo_address_lookup_sf)
export(geo_amenity)
export(geo_amenity_sf)
export(geo_lite)
export(geo_lite_sf)
export(geo_lite_struct)
export(geo_lite_struct_sf)
export(nominatim_check_access)
export(reverse_geo_lite)
export(reverse_geo_lite_sf)
importFrom(utils,download.file)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
