list(
  rd_family_title = list(
    amenity = "Amenity lookup functions and data:",
    geocoding = "Address search functions:",
    lookup = "Address lookup functions:",
    reverse = "Reverse geocoding functions:",
    spatial = "Spatial helper and output functions:"
  )
)
